#include "php_gtk.h"
#if HAVE_PHP_GTK

#include <glade/glade.h>
#include "ext/gtk+/php_gtk+.h"

void php_libglade_register_constants(int module_number TSRMLS_DC)
{
}

PHP_GTK_EXPORT_CE(glade_xml_ce);

PHP_FUNCTION(glade_xml_new_from_memory)
{
	char *buffer, *root, *domain;
	long size;

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "siss", &buffer, &size, &root, &domain))
		return;

	PHP_GTK_SEPARATE_RETURN(return_value, php_gtk_new((GtkObject *)glade_xml_new_from_memory(buffer, (int)size, root, domain)));
	return;
}


PHP_FUNCTION(glade_get_widget_name)
{
	zval *widget;
	const gchar *ret;

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "O", &widget, gtk_widget_ce))
		return;

	ret = glade_get_widget_name(GTK_WIDGET(PHP_GTK_GET(widget)));
	if (ret) {
		RETURN_STRING((char *)ret, 1);
	}
	else {
		RETURN_NULL();
	}
}


PHP_FUNCTION(glade_get_widget_long_name)
{
	zval *widget;
	const gchar *ret;

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "O", &widget, gtk_widget_ce))
		return;

	ret = glade_get_widget_long_name(GTK_WIDGET(PHP_GTK_GET(widget)));
	if (ret) {
		RETURN_STRING((char *)ret, 1);
	}
	else {
		RETURN_NULL();
	}
}


PHP_FUNCTION(glade_get_widget_tree)
{
	zval *widget;

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "O", &widget, gtk_widget_ce))
		return;

	PHP_GTK_SEPARATE_RETURN(return_value, php_gtk_new((GtkObject *)glade_get_widget_tree(GTK_WIDGET(PHP_GTK_GET(widget)))));
	return;
}

PHP_GTK_EXPORT_CE(libglade_ce);

static function_entry php_libglade_functions[] = {
	{"xml_new_from_memory",	PHP_FN(glade_xml_new_from_memory),	NULL},
	{"get_widget_name",	PHP_FN(glade_get_widget_name),	NULL},
	{"get_widget_long_name",	PHP_FN(glade_get_widget_long_name),	NULL},
	{"get_widget_tree",	PHP_FN(glade_get_widget_tree),	NULL},
	{NULL, NULL, NULL}
};


/* object GladeXML  */

PHP_FUNCTION(glade_xml_new_with_domain)
{
	char *fname, *root = NULL, *domain = NULL;
	GtkObject *wrapped_obj;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "s|ss", &fname, &root, &domain)) {
		php_gtk_invalidate(this_ptr);
		return;
	}

	wrapped_obj = (GtkObject *)glade_xml_new_with_domain(fname, root, domain);
	if (!wrapped_obj) {
		php_error(E_WARNING, "%s(): could not create GladeXML object",
				  get_active_function_name(TSRMLS_C));
		php_gtk_invalidate(this_ptr);
		return;
	}

	php_gtk_object_init(wrapped_obj, this_ptr);

}


PHP_FUNCTION(glade_xml_get_type)
{
	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), ""))
		return;

	RETURN_LONG(glade_xml_get_type());
}


PHP_FUNCTION(glade_xml_construct)
{
	char *fname, *root, *domain;
	gboolean php_retval;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "sss", &fname, &root, &domain))
		return;

	php_retval = glade_xml_construct(GLADE_XML(PHP_GTK_GET(this_ptr)), fname, root, domain);
	RETURN_BOOL(php_retval);
}

static void glade_connect_one(const gchar *handler_name, GtkObject *obj, const
							  gchar *signal_name, const gchar *signal_data,
							  GtkObject *connect_object, gboolean after,
							  zval *callback_data)
{
	zval **callback = NULL, **extra = NULL, **pass_object = NULL;
	zval **callback_filename = NULL, **callback_lineno = NULL;
	zval *object;
	TSRMLS_FETCH();

	zend_hash_index_find(Z_ARRVAL_P(callback_data), 0, (void **)&callback);
	zend_hash_index_find(Z_ARRVAL_P(callback_data), 1, (void **)&extra);
	zend_hash_index_find(Z_ARRVAL_P(callback_data), 2, (void **)&pass_object);
	zend_hash_index_find(Z_ARRVAL_P(callback_data), 3, (void **)&callback_filename);
	zend_hash_index_find(Z_ARRVAL_P(callback_data), 4, (void **)&callback_lineno);

	if (connect_object) {
		zval *temp;

		Z_LVAL_PP(pass_object) = 0;
		object = php_gtk_new(connect_object);
		MAKE_STD_ZVAL(temp);
		array_init(temp);
		add_next_index_zval(temp, object);
		php_array_merge(Z_ARRVAL_P(temp), Z_ARRVAL_PP(extra), 0 TSRMLS_CC);
		REPLACE_ZVAL_VALUE(extra, temp, 0);
	}

	gtk_signal_connect_full(obj, signal_name, NULL,
							(GtkCallbackMarshal)php_gtk_callback_marshal,
							callback_data, php_gtk_destroy_notify, FALSE, after);
}

static void glade_signal_connect_impl(INTERNAL_FUNCTION_PARAMETERS, int pass_object)
{
	char *handler_name = NULL;
	zval *callback = NULL;
	zval *extra;
	zval *data;
	char *callback_filename;
	uint callback_lineno;

	NOT_STATIC_METHOD();

	if (ZEND_NUM_ARGS() < 2) {
		php_error(E_WARNING, "%s() requires at least 2 arguments, %d given",
				  get_active_function_name(TSRMLS_C), ZEND_NUM_ARGS());
		return;
	}

	if (!php_gtk_parse_args(2, "sV", &handler_name, &callback))
		return;

	callback_filename = zend_get_executed_filename(TSRMLS_C);
	callback_lineno = zend_get_executed_lineno(TSRMLS_C);
	extra = php_gtk_func_args_as_hash(ZEND_NUM_ARGS(), 2, ZEND_NUM_ARGS());
	data = php_gtk_build_value("(VNisi)", callback, extra, pass_object, callback_filename, callback_lineno);
	glade_xml_signal_connect_full(GLADE_XML(PHP_GTK_GET(this_ptr)), handler_name,
								  (GladeXMLConnectFunc)glade_connect_one, data);
	RETURN_NULL();
}

PHP_FUNCTION(glade_xml_signal_connect)
{
	glade_signal_connect_impl(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
}

typedef struct _php_gtk_autoconnect_data {
	zend_bool pass_object;
	zval *map;
} php_gtk_autoconnect_data;

static void glade_connect_auto(const gchar *handler_name, GtkObject *obj,
							   const gchar *signal_name, const gchar *signal_data,
							   GtkObject *connect_object, gboolean after,
							   php_gtk_autoconnect_data *ac_data)
{
	zval **callback_data = NULL, *map = ac_data->map;
	zval **callback_ptr = NULL, *extra = NULL, *params = NULL;
	zval *callback, *data, *object = NULL;
	int pass_object = ac_data->pass_object;
	char *callback_filename;
	uint callback_lineno;
	TSRMLS_FETCH();

	if (map && zend_hash_find(Z_ARRVAL_P(map), (char *)handler_name, strlen(handler_name) + 1, (void **)&callback_data) == SUCCESS) {
		if (Z_TYPE_PP(callback_data) != IS_ARRAY ||
			zend_hash_index_find(Z_ARRVAL_PP(callback_data), 0, (void **)&callback_ptr) == FAILURE) {
			php_error(E_WARNING, "%s() is supplied with invalid callback structure for handler '%s'", get_active_function_name(TSRMLS_C), handler_name);
			return; 
		}
		zval_add_ref(callback_ptr);
		callback = *callback_ptr;
		zend_hash_index_del(Z_ARRVAL_PP(callback_data), 0);
		extra = *callback_data;
	} else {
		MAKE_STD_ZVAL(callback);
		ZVAL_STRING(callback, (char *)handler_name, 1);
	}

	if (!zend_is_callable(callback, 0, NULL)) {
		php_error(E_WARNING, "%s() is unable to autoconnect callback for handler '%s'",
				  get_active_function_name(TSRMLS_C), handler_name);
		return;
	}
	
	MAKE_STD_ZVAL(params);
	array_init(params);

	if (connect_object) {
		pass_object = 0;
		object = php_gtk_new(connect_object);
		add_next_index_zval(params, object);
	}
	
	if (extra)
		php_array_merge(Z_ARRVAL_P(params), Z_ARRVAL_P(extra), 0 TSRMLS_CC);

	callback_filename = zend_get_executed_filename(TSRMLS_C);
	callback_lineno = zend_get_executed_lineno(TSRMLS_C);
	data = php_gtk_build_value("(NNisi)", callback, params, pass_object,
							   callback_filename, callback_lineno);
	gtk_signal_connect_full(obj, signal_name, NULL,
							(GtkCallbackMarshal)php_gtk_callback_marshal,
							data, php_gtk_destroy_notify, FALSE, after);
}

static void glade_signal_autoconnect_impl(INTERNAL_FUNCTION_PARAMETERS, int pass_object)
{
	zval *map = NULL;
	php_gtk_autoconnect_data *ac_data;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "|a", &map))
		return;

	ac_data = (php_gtk_autoconnect_data *)emalloc(sizeof(php_gtk_autoconnect_data));
	ac_data->map = map;
	ac_data->pass_object = pass_object;
	glade_xml_signal_autoconnect_full(GLADE_XML(PHP_GTK_GET(this_ptr)),
									  (GladeXMLConnectFunc)glade_connect_auto, ac_data);
}

PHP_FUNCTION(glade_xml_signal_autoconnect)
{
	glade_signal_autoconnect_impl(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
}

PHP_FUNCTION(glade_xml_get_widget)
{
	char *name;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "s", &name))
		return;

	PHP_GTK_SEPARATE_RETURN(return_value, php_gtk_new((GtkObject *)glade_xml_get_widget(GLADE_XML(PHP_GTK_GET(this_ptr)), name)));
	return;
}


PHP_FUNCTION(glade_xml_get_widget_by_long_name)
{
	char *longname;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "s", &longname))
		return;

	PHP_GTK_SEPARATE_RETURN(return_value, php_gtk_new((GtkObject *)glade_xml_get_widget_by_long_name(GLADE_XML(PHP_GTK_GET(this_ptr)), longname)));
	return;
}


PHP_FUNCTION(glade_xml_relative_file)
{
	char *filename;
	gchar *ret;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "s", &filename))
		return;

	ret = glade_xml_relative_file(GLADE_XML(PHP_GTK_GET(this_ptr)), filename);
	if (ret) {
		RETURN_STRING(ret, 1);
		g_free(ret);
	} else
		RETURN_NULL();
}

PHP_FUNCTION(glade_xml_signal_connect_object)
{
	glade_signal_connect_impl(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
}
PHP_FUNCTION(glade_xml_signal_autoconnect_object)
{
	glade_signal_autoconnect_impl(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
}

static function_entry php_glade_xml_functions[] = {
	{"GladeXML",	PHP_FN(glade_xml_new_with_domain),	NULL},
	{"gladexml",	PHP_FN(glade_xml_new_with_domain),	NULL},
	{"get_type",	PHP_FN(glade_xml_get_type),	NULL},
	{"construct",	PHP_FN(glade_xml_construct),	NULL},
	{"signal_connect",	PHP_FN(glade_xml_signal_connect),	NULL},
	{"signal_autoconnect",	PHP_FN(glade_xml_signal_autoconnect),	NULL},
	{"get_widget",	PHP_FN(glade_xml_get_widget),	NULL},
	{"get_widget_by_long_name",	PHP_FN(glade_xml_get_widget_by_long_name),	NULL},
	{"relative_file",	PHP_FN(glade_xml_relative_file),	NULL},
	{"signal_connect_object",	PHP_FN(glade_xml_signal_connect_object),	NULL},
	{"signal_autoconnect_object",	PHP_FN(glade_xml_signal_autoconnect_object),	NULL},
	{NULL, NULL, NULL}
};


void php_libglade_register_classes(void)
{
	zend_class_entry ce;
	TSRMLS_FETCH();

	INIT_OVERLOADED_CLASS_ENTRY(ce, "libglade", php_libglade_functions, NULL, NULL, php_gtk_set_property);
	libglade_ce = zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);

	INIT_OVERLOADED_CLASS_ENTRY(ce, "GladeXML", php_glade_xml_functions, NULL, NULL, php_gtk_set_property);
	glade_xml_ce = zend_register_internal_class_ex(&ce, gtk_data_ce, NULL TSRMLS_CC);
	g_hash_table_insert(php_gtk_class_hash, g_strdup("GtkXML"), glade_xml_ce);


}

#endif /* HAVE_PHP_GTK */
