#include "php_gtk.h"
#if HAVE_PHP_GTK

#include "gtkpiemenu.h"
#include "ext/gtk+/php_gtk+.h"

void php_piemenu_register_constants(int module_number TSRMLS_DC)
{

	/* enum GtkPieCompass */
	REGISTER_LONG_CONSTANT("GTK_PIE_N", GTK_PIE_N, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_NNE", GTK_PIE_NNE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_NE", GTK_PIE_NE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_NEE", GTK_PIE_NEE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_E", GTK_PIE_E, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_SEE", GTK_PIE_SEE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_SE", GTK_PIE_SE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_SSE", GTK_PIE_SSE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_S", GTK_PIE_S, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_SSW", GTK_PIE_SSW, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_SW", GTK_PIE_SW, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_SWW", GTK_PIE_SWW, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_W", GTK_PIE_W, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_NWW", GTK_PIE_NWW, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_NW", GTK_PIE_NW, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("GTK_PIE_NNW", GTK_PIE_NNW, CONST_CS | CONST_PERSISTENT);


}

PHP_GTK_EXPORT_CE(gtk_piemenu_ce);

/* object GtkPieMenu  */

PHP_FUNCTION(gtk_pie_menu_new_with_refinement)
{
	long refinement = 4;
	GtkObject *wrapped_obj;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "|i", &refinement)) {
		php_gtk_invalidate(this_ptr);
		return;
	}

	wrapped_obj = (GtkObject *)gtk_pie_menu_new_with_refinement((gint)refinement);
	if (!wrapped_obj) {
		php_error(E_WARNING, "%s(): could not create GtkPieMenu object",
				  get_active_function_name(TSRMLS_C));
		php_gtk_invalidate(this_ptr);
		return;
	}

	php_gtk_object_init(wrapped_obj, this_ptr);

}


PHP_FUNCTION(gtk_pie_menu_get_type)
{
	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), ""))
		return;

	RETURN_LONG(gtk_pie_menu_get_type());
}


PHP_FUNCTION(gtk_pie_menu_append)
{
	zval *child;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "O", &child, gtk_widget_ce))
		return;

	gtk_pie_menu_append(GTK_PIE_MENU(PHP_GTK_GET(this_ptr)), GTK_WIDGET(PHP_GTK_GET(child)));
	RETURN_NULL();
}


PHP_FUNCTION(gtk_pie_menu_put)
{
	zval *child;
	long position;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "Oi", &child, gtk_widget_ce, &position))
		return;

	gtk_pie_menu_put(GTK_PIE_MENU(PHP_GTK_GET(this_ptr)), GTK_WIDGET(PHP_GTK_GET(child)), (gint)position);
	RETURN_NULL();
}


PHP_FUNCTION(gtk_pie_menu_put_with_compass)
{
	zval *child, *php_compass = NULL;
	GtkPieCompass compass;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "OV", &child, gtk_widget_ce, &php_compass))
		return;

	if (php_compass && !php_gtk_get_simple_enum_value(php_compass, (gint *)&compass)) {
		return;
	}

	gtk_pie_menu_put_with_compass(GTK_PIE_MENU(PHP_GTK_GET(this_ptr)), GTK_WIDGET(PHP_GTK_GET(child)), compass);
	RETURN_NULL();
}


PHP_FUNCTION(gtk_pie_menu_put_in_refinement)
{
	zval *child;
	long position, refinement;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "Oii", &child, gtk_widget_ce, &position, &refinement))
		return;

	gtk_pie_menu_put_in_refinement(GTK_PIE_MENU(PHP_GTK_GET(this_ptr)), GTK_WIDGET(PHP_GTK_GET(child)), (gint)position, (gint)refinement);
	RETURN_NULL();
}


PHP_FUNCTION(gtk_pie_menu_set_refinement)
{
	long refinement;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "i", &refinement))
		return;

	gtk_pie_menu_set_refinement(GTK_PIE_MENU(PHP_GTK_GET(this_ptr)), (gint)refinement);
	RETURN_NULL();
}


PHP_FUNCTION(gtk_pie_menu_set_spacing)
{
	long spacing;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "i", &spacing))
		return;

	gtk_pie_menu_set_spacing(GTK_PIE_MENU(PHP_GTK_GET(this_ptr)), (gint)spacing);
	RETURN_NULL();
}


PHP_FUNCTION(gtk_pie_menu_set_radius)
{
	long radius;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "i", &radius))
		return;

	gtk_pie_menu_set_radius(GTK_PIE_MENU(PHP_GTK_GET(this_ptr)), (gint)radius);
	RETURN_NULL();
}


PHP_FUNCTION(gtk_pie_menu_popup)
{
	long button, activate_time;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "ii", &button, &activate_time))
		return;

	gtk_pie_menu_popup(GTK_PIE_MENU(PHP_GTK_GET(this_ptr)), (guint)button, (guint32)activate_time);
	RETURN_NULL();
}


PHP_FUNCTION(gtk_pie_menu_set_title)
{
	zval *child;

	NOT_STATIC_METHOD();

	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "O", &child, gtk_widget_ce))
		return;

	gtk_pie_menu_set_title(GTK_PIE_MENU(PHP_GTK_GET(this_ptr)), GTK_WIDGET(PHP_GTK_GET(child)));
	RETURN_NULL();
}


static function_entry php_gtk_piemenu_functions[] = {
	{"GtkPieMenu",	PHP_FN(gtk_pie_menu_new_with_refinement),	NULL},
	{"gtkpiemenu",	PHP_FN(gtk_pie_menu_new_with_refinement),	NULL},
	{"get_type",	PHP_FN(gtk_pie_menu_get_type),	NULL},
	{"append",	PHP_FN(gtk_pie_menu_append),	NULL},
	{"put",	PHP_FN(gtk_pie_menu_put),	NULL},
	{"put_with_compass",	PHP_FN(gtk_pie_menu_put_with_compass),	NULL},
	{"put_in_refinement",	PHP_FN(gtk_pie_menu_put_in_refinement),	NULL},
	{"set_refinement",	PHP_FN(gtk_pie_menu_set_refinement),	NULL},
	{"set_spacing",	PHP_FN(gtk_pie_menu_set_spacing),	NULL},
	{"set_radius",	PHP_FN(gtk_pie_menu_set_radius),	NULL},
	{"popup",	PHP_FN(gtk_pie_menu_popup),	NULL},
	{"set_title",	PHP_FN(gtk_pie_menu_set_title),	NULL},
	{NULL, NULL, NULL}
};


void php_piemenu_register_classes(void)
{
	zend_class_entry ce;
	TSRMLS_FETCH();

	INIT_OVERLOADED_CLASS_ENTRY(ce, "GtkPieMenu", php_gtk_piemenu_functions, NULL, NULL, php_gtk_set_property);
	gtk_piemenu_ce = zend_register_internal_class_ex(&ce, gtk_menu_ce, NULL TSRMLS_CC);
	g_hash_table_insert(php_gtk_class_hash, g_strdup("GtkPieMenu"), gtk_piemenu_ce);


}

#endif /* HAVE_PHP_GTK */
